package com.yizhi.message.application.service.impl;

import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.plugins.Page;
import com.baomidou.mybatisplus.service.impl.ServiceImpl;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.core.application.event.EventWrapper;
import com.yizhi.core.application.publish.CloudEventPublisher;
import com.yizhi.message.application.constans.Constans;
import com.yizhi.message.application.controller.util.MessageUtil;
import com.yizhi.message.application.controller.util.QueryAccount;
import com.yizhi.message.application.domain.*;
import com.yizhi.message.application.enums.MessageStatus;
import com.yizhi.message.application.event.MessageEventHandler;
import com.yizhi.message.application.mapper.MessageMapper;
import com.yizhi.message.application.service.*;
import com.yizhi.message.application.vo.*;
import com.yizhi.system.application.vo.AccountVO;
import com.yizhi.system.application.vo.OrgVO;
import org.apache.commons.collections.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import java.util.stream.Collectors;


/**
 * <p>
 * 消息 服务实现类
 * </p>
 *
 * @author hutao123
 * @since 2019-09-09
 */
@Service
public class MessageServiceImple extends ServiceImpl<MessageMapper, Message> implements MessageService {


    @Autowired
    private IdGenerator idGenerator;
    @Autowired
    private MessageService messageService;

    @Autowired
    private MessageContextService messageContextService;

    @Autowired
    private MessageVisibleRangeService messageVisibleRangeService;

    @Autowired
    private MessageUtil messageUtil;

    @Autowired
    private MessageAccountService messageAccountService;

    @Autowired
    private MessageRemindService messageRemindService;

    @Autowired
    private QueryAccount queryAccount;


    @Autowired
    private MessageParamRelationService messageParamRelationService;

    @Autowired
    private MessageParameterService messageParameterService;
    @Autowired
    private MessageEventHandler messageEventHandler;

    private static final Logger logger = LoggerFactory.getLogger(MessageServiceImple.class);

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer save(MessageVo messageVo) {


        try {
            RequestContext context = ContextHolder.get();
            Date date = new Date();
            Long messageId = idGenerator.generate();
            //插入主体消息
            Message message = null;
            Integer i = null;
            boolean y = false;
            try {
                message = new Message();
                BeanUtils.copyProperties(messageVo, message);
                message.setId(messageId);
                if (messageVo.getMessageType() != 1) {
                    message.setStatus(MessageStatus.DOWN.getCode());
                } else {
                    message.setStatus(MessageStatus.DRAFT.getCode());
                }
                message.setDeleted(Constans.NOT_DELETED);
                message.setCreateById(context.getAccountId());
                message.setCreateByName(context.getAccountName());
                message.setCreateTime(date);
                message.setCompanyId(context.getCompanyId());
                message.setSiteId(context.getSiteId());
                i = this.baseMapper.insert(message);

                //把消息内容插入
                MessageContext messageContext = new MessageContext();
                messageContext.setId(idGenerator.generate());
                messageContext.setContext(messageVo.getContext());
                messageContext.setMessageId(message.getId());
                messageContext.setCreateById(context.getAccountId());
                messageContext.setCreateByName(context.getAccountName());
                messageContext.setCreateTime(date);
                y = messageContextService.insert(messageContext);

            } catch (Exception e) {
                e.printStackTrace();
                logger.info("保存主体消息异常");
                throw e;
            }


            try {
                //若为自定义消息 且指定可见范围，则插入数据
                List<MessageVisibleRangeVo> visibleRangeVos = messageVo.getMessageVisibleRange();
                boolean b = false;
                if (messageVo.getMessageType() == 1) {
                    if (messageVo.getVisibleRange().equals(Constans.VISIBLE_RANGE)
                            && CollectionUtils.isNotEmpty(visibleRangeVos)) {
                        insertVisibleRange(visibleRangeVos, messageId, context, date);
                    }
                }
            } catch (Exception e) {
                e.printStackTrace();
                logger.info("保存可见范围异常");
                throw e;
            }
            try {
                //插入消息与可选参数的关联关系
                insertMessageParamRelation(messageId, messageVo.getMessageParameterIds(), context, date);
            } catch (Exception e) {
                e.printStackTrace();
                logger.info("保存可选参数关联关系异常");
                throw e;
            }


            if (i == 1 && y == true) {
                return 1;
            } else if (i != 1) {
                return -2;//保存主体消息异常
            } else if (y != true) {
                return -3;//保存消息内容异常
            }
        } catch (Exception e) {
            e.printStackTrace();
            throw e;
        }
        return -1;//消息保存失败
    }

    @Override
    public Page<MessageManageVo> getPage(Integer messageType, String title, Page<MessageManageVo> page) {


        List<MessageManageVo> messageVoList = this.baseMapper.getPage(messageType, title, ContextHolder.get().getSiteId(), page);

        //如果是触发消息没有查到当前站点的触发消息，则取初始化消息(siteID为0)
        if (messageType == 3 && CollectionUtils.isEmpty(messageVoList)) {
            initData(ContextHolder.get(), new Date());
            messageVoList = this.baseMapper.getPage(messageType, title, ContextHolder.get().getSiteId(), page);
            page.setTotal(this.baseMapper.getPageNum(messageType, title, ContextHolder.get().getSiteId()));
        }
        Integer num = page.getOffset() + 1;
        if (CollectionUtils.isNotEmpty(messageVoList)) {

            for (MessageManageVo vo : messageVoList) {
                vo.setNum(num);
                num++;
                //设置
                vo.setRelationTypeString(messageUtil.getRelationType(vo.getRelationType()));
                vo.setStatusString(messageUtil.getMessageStatus(vo.getStatus()));
            }
        }
        page.setRecords(messageVoList);

        return page;
    }

    @Override
    public MessageVo get(Long id) {
        Message message = this.baseMapper.selectById(id);
        MessageVo messageVo = new MessageVo();
        if (message != null) {
            BeanUtils.copyProperties(message, messageVo);
            EntityWrapper wrapper = new EntityWrapper();
            wrapper.eq("message_id", id);
            MessageContext messageContext = messageContextService.selectOne(wrapper);
            if (messageContext != null) {
                messageVo.setContext(messageContext.getContext());
                messageVo.setFakeContext(messageUtil.returnParameter(message.getRelationType(), messageContext.getContext()));
                messageVo.setMessageContextId(messageContext.getId());
            }
            //获取消息已选的参数列表
            EntityWrapper<MessageParamRelation> wrapper1 = new EntityWrapper<>();
            wrapper1.eq("message_id", message.getId());
            wrapper1.eq("deleted", Constans.NOT_DELETED);
            List<MessageParamRelation> relations = messageParamRelationService.selectList(wrapper1);
            if (CollectionUtils.isNotEmpty(relations)) {
                List<Long> messageParameterIds = new ArrayList<>(relations.size());
                relations.forEach(a -> {
                    messageParameterIds.add(a.getMessageParameterId());
                });
                messageVo.setMessageParameterIds(messageParameterIds);
            }


            //自定义消息需要回显可见范围   需要实时获取真实姓名等功能
            if (message.getMessageType() == 1) {
                Map<String, Object> map = new HashMap<>(2);
                map.put("message_id", message.getId());
                List<MessageVisibleRange> visibleRanges = messageVisibleRangeService.selectByMap(map);

                if (CollectionUtils.isNotEmpty(visibleRanges)) {
                    List<Long> accountIds = new ArrayList<>(visibleRanges.size());
                    List<Long> orgIds = new ArrayList<>(visibleRanges.size());
                    List<MessageVisibleRangeVo> vos = new ArrayList<>(visibleRanges.size());

                    for (MessageVisibleRange a : visibleRanges) {
                        MessageVisibleRangeVo vo = new MessageVisibleRangeVo();
                        BeanUtils.copyProperties(a, vo);
                        vos.add(vo);
                        if (a.getType().equals(1)) {
                            orgIds.add(a.getRelationId());
                        } else {
                            accountIds.add(a.getRelationId());
                        }
                    }
                    Map<Long, AccountVO> accountMap = queryAccount.queryAccountByIds(accountIds);
                    Map<Long, OrgVO> orgMap = queryAccount.queryOrgByIds(orgIds);
                    for (MessageVisibleRangeVo a : vos) {
                        if (null != accountMap) {
                            AccountVO accountVO = accountMap.get(a.getRelationId());
                            if (null != accountVO && a.getType().equals(2)) {
                                a.setWorkNum(accountVO.getWorkNum());
                                a.setFullName(accountVO.getFullName());
                                a.setName(accountVO.getName());
                            }
                        }
                        if (null != orgMap) {
                            OrgVO orgVO = orgMap.get(a.getRelationId());
                            if (null != orgVO && a.getType().equals(1)) {
                                a.setName(orgVO.getName());
                            }
                        }
                    }
                    messageVo.setMessageVisibleRange(vos);
                }
            } else {
                messageVo.setRelationTypeString(messageUtil.getRelationType(message.getRelationType()));
            }
        }
        return messageVo;
    }

    /**
     * 插入可见范围
     *
     * @param visibleRangeVos
     * @param messageId
     * @param requestContext
     * @param date
     * @return
     */
    public Boolean insertVisibleRange(List<MessageVisibleRangeVo> visibleRangeVos, Long messageId, RequestContext requestContext, Date date) {
        List<MessageVisibleRange> visibleRanges = new ArrayList<>(visibleRangeVos.size());
        for (MessageVisibleRangeVo vo : visibleRangeVos) {

            MessageVisibleRange visibleRange = new MessageVisibleRange();
            visibleRange.setId(idGenerator.generate());
            visibleRange.setType(vo.getType());
            visibleRange.setMessageId(messageId);
            visibleRange.setRelationId(vo.getRelationId());
            visibleRange.setCreateById(requestContext.getAccountId());
            visibleRange.setCreateByName(requestContext.getAccountName());
            visibleRange.setCreateTime(date);
            visibleRanges.add(visibleRange);
        }
        return messageVisibleRangeService.insertBatch(visibleRanges);
    }

    /**
     * 清空可见范围
     *
     * @param messageId
     * @param requestContext
     * @param date
     * @return
     */
    public boolean deleteVisibleRange(Long messageId, RequestContext requestContext, Date date) {
        EntityWrapper wrapper = new EntityWrapper();
        wrapper.eq("message_id", messageId);
        MessageVisibleRange visibleRange = new MessageVisibleRange();
        visibleRange.setDeleted(Constans.DELETED);
        visibleRange.setUpdateById(requestContext.getAccountId());
        visibleRange.setUpdateByName(requestContext.getAccountName());
        visibleRange.setUpdateTime(date);
        return this.messageVisibleRangeService.update(visibleRange, wrapper);
    }

    /**
     * 插入消息 与可选参数的关联关系
     *
     * @param messageid
     * @param messageParameterIds
     * @param context
     * @param date
     * @return
     */
    public boolean insertMessageParamRelation(Long messageid, List<Long> messageParameterIds, RequestContext context, Date date) {
        if (CollectionUtils.isNotEmpty(messageParameterIds)) {
            List<MessageParamRelation> relations = new ArrayList<>();
            for (Long parameterId : messageParameterIds) {
                MessageParameter parameter = messageParameterService.selectById(parameterId);
                if (parameter != null) {
                    MessageParamRelation relation = new MessageParamRelation();
                    relation.setId(idGenerator.generate());
                    relation.setMessageId(messageid);
                    relation.setFieldType(parameter.getFieldType());
                    relation.setMessageParameterId(parameterId);
                    relation.setCreateById(context.getAccountId());
                    relation.setCreateByName(context.getAccountName());
                    relation.setCreateTime(date);
                    relations.add(relation);
                } else {
                    logger.info("parameterId:" + parameterId + "--数据库没有该参数对象");
                }
            }
            if (CollectionUtils.isNotEmpty(relations)) {
                return messageParamRelationService.insertBatch(relations);
            }
        }
        return false;
    }

    /**
     * 清空消息与可选参数的关联关系
     *
     * @param messageId
     * @param context
     * @param date
     * @return
     */
    public boolean deleteMessageParamRelation(Long messageId, RequestContext context, Date date) {

        EntityWrapper wrapper = new EntityWrapper();
        wrapper.eq("message_id", messageId);
        MessageParamRelation messageParamRelation = new MessageParamRelation();
        messageParamRelation.setDeleted(Constans.DELETED);
        messageParamRelation.setUpdateById(context.getAccountId());
        messageParamRelation.setUpdateByName(context.getAccountName());
        messageParamRelation.setUpdateTime(date);

        return messageParamRelationService.update(messageParamRelation, wrapper);

    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer update(MessageVo messageVo) {

        if (messageVo != null) {
            Date date = new Date();
            RequestContext requestContext = ContextHolder.get();
            MessageContext context = new MessageContext();
            Message tem = this.baseMapper.selectById(messageVo.getId());
            if (tem == null) {
                return -3;
            }
            Message message = new Message();
            BeanUtils.copyProperties(messageVo, message);
            Long updateMessageId = idGenerator.generate();

            boolean b = false;
            boolean canUpdate = false;
            if ((messageVo.getId() >= 10 || messageVo.getId() <= 1)) {
                canUpdate = true;
            }

            try {

                try {
                    //如果是自定义消息，判断可见范围是否有变化
                    if (tem.getMessageType() == 1) {
                        List<MessageVisibleRangeVo> visibleRangeVos = messageVo.getMessageVisibleRange();
                        if (!MessageStatus.DRAFT.getCode().equals(tem.getStatus())) {
                            return -4;
                        }
                        if (Constans.VISIBLE_RANGE.equals(messageVo.getVisibleRange())
                                && CollectionUtils.isNotEmpty(visibleRangeVos)) {
                            deleteVisibleRange(messageVo.getId(), requestContext, date);
                            insertVisibleRange(visibleRangeVos, messageVo.getId(), requestContext, date);
                        } else {
                            deleteVisibleRange(messageVo.getId(), requestContext, date);
                        }
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                    logger.info("保存可见范围异常");
                    throw e;
                }
                try {
                    //消息的可选参数发生改变
                    if (CollectionUtils.isNotEmpty(messageVo.getMessageParameterIds())) {
                        //防止误修改默认触发消息
                        if (canUpdate) {
                            deleteMessageParamRelation(messageVo.getId(), requestContext, date);
                            insertMessageParamRelation(messageVo.getId(), messageVo.getMessageParameterIds(), requestContext, date);
                        }
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                    logger.info("保存可选参数关联关系异常");
                    throw e;
                }
                //防止误修改默认触发消息
                if (canUpdate) {
                    b = message.updateById();
                }


                try {
                    //消息内容发生改变
                    if (messageVo.getContext() != null) {

                        //防止误修改默认触发消息
                        if (canUpdate) {
                            context.setId(messageVo.getMessageContextId());
                            context.setContext(messageVo.getContext());
                            boolean c =false;
                            if(messageVo.getMessageContextId()==null){
                                context.setId(idGenerator.generate());
                                context.setMessageId(message.getId());
                                context.setCreateById(requestContext.getAccountId());
                                context.setCreateByName(requestContext.getAccountName());
                                context.setCreateTime(date);
                                c = messageContextService.insert(context);
                            }else {
                                c = context.updateById();
                            }
                        }
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                    logger.info("保存消息内容异常");
                    throw e;
                }
            } catch (Exception e) {
                e.printStackTrace();
                logger.info("修改消息内容异常");
                throw e;
            }
            if (b) {
                return 1;
            }
        }
        return -1;//修改失败
    }

    public void initData(RequestContext requestContext, Date date) {

        //如果id在1-10之间，说明此站点需要新增自己的触发消息
        List<Message> messages = new ArrayList<>();

        messages = this.baseMapper.getList(3, 0L);
        if (CollectionUtils.isNotEmpty(messages)) {
            for (Message a : messages) {
                Long oldMessageId = a.getId();
                Long messageId = idGenerator.generate();

                //把消息内容插入
                MessageContext messageContext1 = new MessageContext();
                messageContext1.setMessageId(oldMessageId);
                EntityWrapper<MessageContext> wrapper = new EntityWrapper<>(messageContext1);
                messageContext1 = messageContextService.selectOne(wrapper);
                if (messageContext1 != null) {
                    messageContext1.setId(idGenerator.generate());
                    messageContext1.setContext(messageContext1.getContext());
                    messageContext1.setMessageId(messageId);
                    messageContext1.setCreateById(requestContext.getAccountId());
                    messageContext1.setCreateByName(requestContext.getAccountName());
                    messageContext1.setCreateTime(date);
                    messageContextService.insert(messageContext1);
                }
                //插入消息与可选参数的关联关系
                MessageParamRelation relation = new MessageParamRelation();
                relation.setDeleted(0);
                relation.setMessageId(oldMessageId);
                EntityWrapper<MessageParamRelation> wrapper1 = new EntityWrapper<>(relation);
                List<MessageParamRelation> relations = messageParamRelationService.selectList(wrapper1);
                if (CollectionUtils.isNotEmpty(relations)) {
                    for (MessageParamRelation paramRelation : relations) {
                        paramRelation.setId(idGenerator.generate());
                        paramRelation.setMessageId(messageId);
                        paramRelation.setCreateById(requestContext.getAccountId());
                        paramRelation.setCreateByName(requestContext.getAccountName());
                        paramRelation.setCreateTime(date);
                    }
                    messageParamRelationService.insertBatch(relations);
                }
                //插入主体消息
                a.setId(messageId);
                a.setSiteId(requestContext.getSiteId());
                a.setDeleted(Constans.NOT_DELETED);
                a.setCompanyId(requestContext.getCompanyId());
                a.setCreateById(requestContext.getAccountId());
                a.setCreateByName(requestContext.getAccountName());
                a.setCreateTime(date);
                a.setStatus(MessageStatus.DOWN.getCode());
            }
            messageService.insertBatch(messages);
        }
    }

    @Override
    public Integer upOrDown(CancelSendMessageVo vo) {
        if (vo != null) {
            Message tem = this.baseMapper.selectById(vo.getId());
            if (tem != null) {
                if (tem.getMessageType() != 1) {
                    Message message = new Message();
                    message.setId(vo.getId());
                    message.setStatus(vo.getStatus());
                    if (message.updateById()) {
                        return 1;
                    }
                } else {
                    return -2;//该消息不支持上下架
                }
            }
        }
        return -1;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer cancel(CancelSendMessageVo vo) {

        Message tem = this.baseMapper.selectById(vo.getId());

        if (tem != null) {
            try {
                //只有自定义消息才有取消与撤回功能
                if (tem.getMessageType() == 1) {
                    Message message = new Message();
                    message.setId(vo.getId());

                    //查询是否存在对应待发消息
                    EntityWrapper wrapper = new EntityWrapper();
                    wrapper.eq("message_id", message.getId());
                    MessageRemind messageRemind = messageRemindService.selectOne(wrapper);

                    if (messageRemind != null) {

                        //取消发送操作，更新自定义消息的状态
                        if (vo.getType() == 1) {
                            //顺便更新一下主体消息状态
                            message.setStatus(MessageStatus.DRAFT.getCode());

                            //删除待发消息
                            try {
                                messageRemind.setDeleted(Constans.DELETED);
                                messageRemind.setStatus(MessageStatus.DRAFT.getCode());
                                messageRemindService.updateById(messageRemind);
                            } catch (Exception e) {
                                e.printStackTrace();
                                logger.error("删除待发消息异常" + "错误消息={}", e);
                                throw e;
                            }
                        }
                        //撤回操作
                        if (vo.getType().equals(2)) {
                            RequestContext context = ContextHolder.get();

                            //更新主体消息状态为已发送被撤回
                            message.setStatus(MessageStatus.SEND_ED_CANCEl.getCode());

                            //删除用户收到的消息
                            try {
                                MessageAccount messageAccount = new MessageAccount();
                                messageAccount.setDeleted(Constans.DELETED);
                                wrapper = new EntityWrapper();
                                wrapper.eq("message_remind_id", messageRemind.getId());
                                wrapper.eq("site_id", context.getSiteId());
                                wrapper.eq("company_id", context.getCompanyId());
                                messageAccountService.update(messageAccount, wrapper);
                            } catch (Exception e) {
                                e.printStackTrace();
                                logger.info("删除用户消息异常");
                                throw e;
                            }

                            try {
                                //更新待发消息状态
                                messageRemind.setStatus(MessageStatus.SEND_ED_CANCEl.getCode());
                                messageRemindService.updateById(messageRemind);
                            } catch (Exception e) {
                                e.printStackTrace();
                                logger.info("更新待发消息异常");
                                throw e;
                            }
                        }
                        //更新主体消息状态
                        if (message.updateById()) {
                            return 1;
                        }
                    } else {
                        logger.info("没有查询到对应待发消息！");
                        return -5;
                    }
                } else {
                    return -2;
                }
            } catch (Exception e) {
                e.printStackTrace();
                logger.info("操作异常");
                throw e;
            }
        } else {
            return -3;
        }
        return -1;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Integer send(CancelSendMessageVo vo) {

        Message message = this.baseMapper.selectById(vo.getId());
        boolean d = false;
        if (message != null) {
            RequestContext context = ContextHolder.get();
            Date date = new Date();

            if (message.getSendTime().getTime() < date.getTime()) {
                return -3;
            }
            //只有自定义消息且草稿状态才能发送
            if (message.getMessageType().equals(1) && MessageStatus.DRAFT.getCode().equals(message.getStatus())) {
                EntityWrapper wrapper = new EntityWrapper();
                wrapper.eq("message_id", message.getId());
                MessageContext messageContext = messageContextService.selectOne(wrapper);


                if (messageContext != null) {
                    String context1 = messageContext.getContext();
                    com.yizhi.message.application.vo.MessageRemindVo vo1 = new com.yizhi.message.application.vo.MessageRemindVo();
                    vo1.setVisibleRange(message.getVisibleRange());
                    vo1.setRelationType(message.getRelationType());
                    vo1.setMessageId(message.getId());
                    vo1.setMessageContext(context1);
                    vo1.setMessageType(message.getMessageType());
                    vo1.setSendType(message.getSendType());
                    vo1.setSendTime(message.getSendTime());
                    vo1.setRequestContext(context);

                    try {
                        messageEventHandler.handle(new EventWrapper<MessageRemindVo>(null, vo1));
                        d = true;
                    } catch (Exception e) {
                        e.printStackTrace();
                        logger.error("发送消息失败=====================", e);
                    }
                    if (d) {
                        //顺便把自定义消息状态改为 待发送
                        message = new Message();
                        message.setId(vo.getId());
                        message.setStatus(MessageStatus.TO_SEND.getCode());
                        message.updateById();
                        return 1;
                    }
                }
            } else {
                return -2;
            }
        } else {
            return -4;
        }
        return -1;
    }

    @Override
    public List<MessageVo> getListByRelationType(Integer relationType) {
        RequestContext context = ContextHolder.get();

        List<MessageVo> messageVoList = new ArrayList<>();
        EntityWrapper wrapper = new EntityWrapper();
        wrapper.eq("deleted", Constans.NOT_DELETED);
        wrapper.eq("status", MessageStatus.UP.getCode());
        wrapper.eq("message_type", 2);
        wrapper.eq("relation_type", relationType)
                .andNew().eq("site_id", context.getSiteId())
                .or().eq("site_id", 0);

        List<Message> messages = this.baseMapper.selectList(wrapper);
        if (CollectionUtils.isNotEmpty(messages)) {
            for (Message message : messages) {
                MessageVo vo = new MessageVo();
                vo.setMessageType(message.getMessageType());
                vo.setId(message.getId());
                vo.setRelationType(message.getRelationType());
                vo.setRelationTypeString(messageUtil.getRelationType(message.getRelationType()));
                vo.setTemplateName(message.getTemplateName());
                vo.setTitle(message.getTitle());
                vo.setEventName(message.getEventName());

                wrapper = new EntityWrapper();
                wrapper.eq("message_id", message.getId());
                MessageContext messageContext = messageContextService.selectOne(wrapper);
                if (messageContext != null) {
                    vo.setContext(messageContext.getContext());
                }
                messageVoList.add(vo);
            }
        }
        return messageVoList;
    }

    @Override
    public Map<Long, String> getMessageTitleMap(List<Long> messageIds) {
        Map<Long, String> map = null;

        if (CollectionUtils.isNotEmpty(messageIds)) {
            List<Message> messages = this.selectBatchIds(messageIds);
            if (CollectionUtils.isNotEmpty(messages)) {
                map = messages.parallelStream().collect(Collectors.toMap(Message::getId, Message::getTitle));
            }
        }
        return map;
    }
}
